#include "DS18B20_temp.h"

OneWire ds18b20(DS18B20_DATA_PIN);

/* -------------------------------------------------------------------
 *  Liefert den Temperaturwert [°C] des DS18B20 mit der Seriennummer
 *  gemäß "ROMcodeIdx" als Integer.
 *  Die unteren vier Bit sind Nachkommastellen - also durch 16.0
 *  teilen, um die Temperatur mit Nachkommastellen zu erhalten.
 */
int readDS18B20(byte ROMcodeIdx) {
  static byte scratchpad[2];    // hier interessieren nur die ersten zwei Byte

  ds18b20.reset();
  ds18b20.select(ROMcode_g[ROMcodeIdx]);
  ds18b20.write(0x44);  // This command initiates a single temperature conversion
  ds18b20.reset();
  ds18b20.select(ROMcode_g[ROMcodeIdx]);
  ds18b20.write(0xBE);  // Read Scratchpad
  for(int i=0; i<2; i++) {
    scratchpad[i]=ds18b20.read();
  }
  // BIT    15 14 1  1  11 10  09  08  07  06  05  04   03   02   01   00
  //        S  S  S  S  S  2^6 2^5 2^4 2^3 2^2 2^1 2^0 2^-1 2^-2 2^-3 2^-4
  return(scratchpad[1]<<8|scratchpad[0]);
}

int readDS18B20_CRC(byte ROMcodeIdx) {
  static byte scratchpad[9]; // 9 Byte

  ds18b20.reset();
  ds18b20.select(ROMcode_g[ROMcodeIdx]);
  ds18b20.write(0x44);  // This command initiates a single temperature conversion
  ds18b20.reset();
  ds18b20.select(ROMcode_g[ROMcodeIdx]);
  ds18b20.write(0xBE);  // Read Scratchpad
  for(int i=0; i<9; i++) {
    scratchpad[i]=ds18b20.read();
  }
  if(OneWire::crc8(scratchpad, 8)!=scratchpad[8]) {
    Serial.println("DS18B20 CRC Fehler");
  }
  // BIT    15 14 1  1  11 10  09  08  07  06  05  04   03   02   01   00
  //        S  S  S  S  S  2^6 2^5 2^4 2^3 2^2 2^1 2^0 2^-1 2^-2 2^-3 2^-4
  return(scratchpad[1]<<8|scratchpad[0]);
}


/* -------------------------------------------------------------------
 *  Setzt "sensor_count_g" auf die Anzahl der angeschlossenen DS18B20
 *  und vermerkt deren Seriennummern in "ROMcode_g[x][]".
 */
char discoverOneWireDevices(void) {
  sensor_count_g=0;
  while(ds18b20.search(ROMcode_g[sensor_count_g])) {
    if(OneWire::crc8(ROMcode_g[sensor_count_g], 7)!=ROMcode_g[sensor_count_g][7]) {
      return(0);
    }
    sensor_count_g++;
    if(sensor_count_g>=MAX_SENSOR_COUNT) { // mehr als zwei Sensor-Paare sind nicht vorgesehen
      break;
    }
  }
  ds18b20.reset_search();
  //sensor_count_g=1;
  return(1);
}

