#include "deep_sleep.h"

//                   0:16ms    1:32ms    2:64ms    3:128ms   4:256ms   5:512ms   6:1024ms  7:2048ms  8:4096ms  9:8192ms
byte sleep_times[]={ 0b000000, 0b000001, 0b000010, 0b000011, 0b000100, 0b000101, 0b000110, 0b000111, 0b100000, 0b100001 };


/* ##################################################################
 * Quelle: http://gammon.com.au/power (Sketch I)
 */

/* -------------------------------------------------------------------
 * Aufwachen
 */
ISR(WDT_vect) {
   wdt_disable();  // disable watchdog
}


/* -------------------------------------------------------------------
 * Schlafen
 *
 *  WDP3  WDP2  WDP1  WDP0
 *   0     0     0     0      16 ms
 *   0     0     0     1      32 ms
 *   0     0     1     0      64 ms
 *   0     0     1     1      0.125 s
 *   0     1     0     0      0.25 s
 *   0     1     0     1      0.5 s
 *   0     1     1     0      1.0 s
 *   0     1     1     1      2.0 s
 *   1     0     0     0      4.0 s
 *   1     0     0     1      8.0 s
 */
void sleep_PWR_DOWN(unsigned char dur=0) {
  // disable ADC
  ADCSRA=0;

  // clear various "reset" flags
  MCUSR=0;     
  // allow changes, disable reset
  WDTCSR=bit(WDCE) | bit(WDE);
  // set interrupt mode and an interval
  WDTCSR=bit(WDIE) | dur;
  wdt_reset();  // pat the dog
 
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);
  noInterrupts();           // timed sequence follows
  sleep_enable();
 
  // turn off brown-out enable in software
  MCUCR=bit(BODS) | bit(BODSE);
  MCUCR=bit(BODS);
  interrupts();             // guarantees next instruction executed
  sleep_cpu();  
 
  // cancel sleep as a precaution
  sleep_disable();
}


/* -------------------------------------------------------------------
 *  ADC wieder aktivieren.
 *  https://www.gammon.com.au/adc
 */
void restore_ADC(void) {
  ADCSRA=bit(ADEN);                         // turn ADC on
  ADCSRA|=bit(ADPS0)|bit(ADPS1)|bit(ADPS2); // Prescaler of 128
}


/* -------------------------------------------------------------------
 *  Wartet "ms" Millisekunden per Tiefschlaf.
 *  Bei "ms" <=0 wird nicht geschlafen.
 */
void deepSleep(int ms) {
  int start_at=8192, idx=9;               // bei der längsten Zeit anfangen

  while(ms>0 && idx>=0) {                 // solange warten noch nötig und möglich ist
    while(ms>=start_at) {                 // solange Wartezeit noch passt
      sleep_PWR_DOWN(sleep_times[idx]);   // aktuelle Zeit warten
      ms-=start_at;                       // Rest-Wartezeit korrigieren
    }
    start_at>>=1;                         // nächst-kleinere Zeit einstellen
    idx--;                                // und Index entsprechend setzen
  }
}

